@@ -1,8 +1,9 @@
 import binascii
 import os
+
 from django.conf import settings
 from django.db import models
-
+from django.utils.encoding import python_2_unicode_compatible
 
 # Prior to Django 1.5, the AUTH_USER_MODEL setting does not exist.
 # Note that we don't perform this code in the compat module due to
@@ -11,6 +12,7 @@ from django.db import models
 AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')
 
 
+@python_2_unicode_compatible
 class Token(models.Model):
     """
     The default authorization token model.
@@ -35,5 +37,5 @@ class Token(models.Model):
     def generate_key(self):
         return binascii.hexlify(os.urandom(20)).decode()
 
-    def __unicode__(self):
+    def __str__(self):
         return self.key
