@@ -19,9 +19,11 @@ automatically.
 from __future__ import unicode_literals
 
 from functools import update_wrapper
+
 from django.utils.decorators import classonlymethod
 from django.views.decorators.csrf import csrf_exempt
-from rest_framework import views, generics, mixins
+
+from rest_framework import generics, mixins, views
 
 
 class ViewSetMixin(object):
@@ -44,10 +46,16 @@ class ViewSetMixin(object):
         instantiated view, we need to totally reimplement `.as_view`,
         and slightly modify the view function that is created and returned.
         """
-        # The suffix initkwarg is reserved for identifing the viewset type
+        # The suffix initkwarg is reserved for identifying the viewset type
         # eg. 'List' or 'Instance'.
         cls.suffix = None
 
+        # actions must not be empty
+        if not actions:
+            raise TypeError("The `actions` argument must be provided when "
+                            "calling `.as_view()` on a ViewSet. For example "
+                            "`.as_view({'get': 'list'})`")
+
         # sanitize keyword arguments
         for key in initkwargs:
             if key in cls.http_method_names:
@@ -92,13 +100,20 @@ class ViewSetMixin(object):
         view.suffix = initkwargs.get('suffix', None)
         return csrf_exempt(view)
 
-    def initialize_request(self, request, *args, **kargs):
+    def initialize_request(self, request, *args, **kwargs):
         """
         Set the `.action` attribute on the view,
         depending on the request method.
         """
-        request = super(ViewSetMixin, self).initialize_request(request, *args, **kargs)
-        self.action = self.action_map.get(request.method.lower())
+        request = super(ViewSetMixin, self).initialize_request(request, *args, **kwargs)
+        method = request.method.lower()
+        if method == 'options':
+            # This is a special case as we always provide handling for the
+            # options method in the base `View` class.
+            # Unlike the other explicitly defined actions, 'metadata' is implict.
+            self.action = 'metadata'
+        else:
+            self.action = self.action_map.get(method)
         return request
 
 
