@@ -1,58 +1,80 @@
 from __future__ import unicode_literals
+
+import django
+import pytest
 from django.db import models
 from django.shortcuts import get_object_or_404
 from django.test import TestCase
 from django.utils import six
+
 from rest_framework import generics, renderers, serializers, status
+from rest_framework.response import Response
 from rest_framework.test import APIRequestFactory
-from tests.models import BasicModel, Comment, SlugBasedModel
-from tests.models import ForeignKeySource, ForeignKeyTarget
+from tests.models import (
+    BasicModel, ForeignKeySource, ForeignKeyTarget, RESTFrameworkModel
+)
 
 factory = APIRequestFactory()
 
 
-class RootView(generics.ListCreateAPIView):
-    """
-    Example description for OPTIONS.
-    """
-    model = BasicModel
+# Models
+class SlugBasedModel(RESTFrameworkModel):
+    text = models.CharField(max_length=100)
+    slug = models.SlugField(max_length=32)
 
 
-class InstanceView(generics.RetrieveUpdateDestroyAPIView):
-    """
-    Example description for OPTIONS.
-    """
-    model = BasicModel
-
-    def get_queryset(self):
-        queryset = super(InstanceView, self).get_queryset()
-        return queryset.exclude(text='filtered out')
+# Model for regression test for #285
+class Comment(RESTFrameworkModel):
+    email = models.EmailField()
+    content = models.CharField(max_length=200)
+    created = models.DateTimeField(auto_now_add=True)
 
 
-class FKInstanceView(generics.RetrieveUpdateDestroyAPIView):
-    """
-    FK: example description for OPTIONS.
-    """
-    model = ForeignKeySource
+# Serializers
+class BasicSerializer(serializers.ModelSerializer):
+    class Meta:
+        model = BasicModel
+
+
+class ForeignKeySerializer(serializers.ModelSerializer):
+    class Meta:
+        model = ForeignKeySource
 
 
 class SlugSerializer(serializers.ModelSerializer):
-    slug = serializers.Field()  # read only
+    slug = serializers.ReadOnlyField()
 
     class Meta:
         model = SlugBasedModel
-        exclude = ('id',)
+        fields = ('text', 'slug')
+
+
+# Views
+class RootView(generics.ListCreateAPIView):
+    queryset = BasicModel.objects.all()
+    serializer_class = BasicSerializer
+
+
+class InstanceView(generics.RetrieveUpdateDestroyAPIView):
+    queryset = BasicModel.objects.exclude(text='filtered out')
+    serializer_class = BasicSerializer
+
+
+class FKInstanceView(generics.RetrieveUpdateDestroyAPIView):
+    queryset = ForeignKeySource.objects.all()
+    serializer_class = ForeignKeySerializer
 
 
 class SlugBasedInstanceView(InstanceView):
     """
     A model with a slug-field.
     """
-    model = SlugBasedModel
+    queryset = SlugBasedModel.objects.all()
     serializer_class = SlugSerializer
     lookup_field = 'slug'
 
 
+# Tests
 class TestRootView(TestCase):
     def setUp(self):
         """
@@ -100,7 +122,7 @@ class TestRootView(TestCase):
         with self.assertNumQueries(0):
             response = self.view(request).render()
         self.assertEqual(response.status_code, status.HTTP_405_METHOD_NOT_ALLOWED)
-        self.assertEqual(response.data, {"detail": "Method 'PUT' not allowed."})
+        self.assertEqual(response.data, {"detail": 'Method "PUT" not allowed.'})
 
     def test_delete_root_view(self):
         """
@@ -110,48 +132,7 @@ class TestRootView(TestCase):
         with self.assertNumQueries(0):
             response = self.view(request).render()
         self.assertEqual(response.status_code, status.HTTP_405_METHOD_NOT_ALLOWED)
-        self.assertEqual(response.data, {"detail": "Method 'DELETE' not allowed."})
-
-    def test_options_root_view(self):
-        """
-        OPTIONS requests to ListCreateAPIView should return metadata
-        """
-        request = factory.options('/')
-        with self.assertNumQueries(0):
-            response = self.view(request).render()
-        expected = {
-            'parses': [
-                'application/json',
-                'application/x-www-form-urlencoded',
-                'multipart/form-data'
-            ],
-            'renders': [
-                'application/json',
-                'text/html'
-            ],
-            'name': 'Root',
-            'description': 'Example description for OPTIONS.',
-            'actions': {
-                'POST': {
-                    'text': {
-                        'max_length': 100,
-                        'read_only': False,
-                        'required': True,
-                        'type': 'string',
-                        "label": "Text comes here",
-                        "help_text": "Text description."
-                    },
-                    'id': {
-                        'read_only': True,
-                        'required': False,
-                        'type': 'integer',
-                        'label': 'ID',
-                    },
-                }
-            }
-        }
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        self.assertEqual(response.data, expected)
+        self.assertEqual(response.data, {"detail": 'Method "DELETE" not allowed.'})
 
     def test_post_cannot_set_id(self):
         """
@@ -166,11 +147,24 @@ class TestRootView(TestCase):
         created = self.objects.get(id=4)
         self.assertEqual(created.text, 'foobar')
 
+    def test_post_error_root_view(self):
+        """
+        POST requests to ListCreateAPIView in HTML should include a form error.
+        """
+        data = {'text': 'foobar' * 100}
+        request = factory.post('/', data, HTTP_ACCEPT='text/html')
+        response = self.view(request).render()
+        expected_error = '<span class="help-block">Ensure this field has no more than 100 characters.</span>'
+        self.assertIn(expected_error, response.rendered_content.decode('utf-8'))
+
+
+EXPECTED_QUERIES_FOR_PUT = 3 if django.VERSION < (1, 6) else 2
+
 
 class TestInstanceView(TestCase):
     def setUp(self):
         """
-        Create 3 BasicModel intances.
+        Create 3 BasicModel instances.
         """
         items = ['foo', 'bar', 'baz', 'filtered out']
         for item in items:
@@ -202,7 +196,7 @@ class TestInstanceView(TestCase):
         with self.assertNumQueries(0):
             response = self.view(request).render()
         self.assertEqual(response.status_code, status.HTTP_405_METHOD_NOT_ALLOWED)
-        self.assertEqual(response.data, {"detail": "Method 'POST' not allowed."})
+        self.assertEqual(response.data, {"detail": 'Method "POST" not allowed.'})
 
     def test_put_instance_view(self):
         """
@@ -210,10 +204,10 @@ class TestInstanceView(TestCase):
         """
         data = {'text': 'foobar'}
         request = factory.put('/1', data, format='json')
-        with self.assertNumQueries(2):
+        with self.assertNumQueries(EXPECTED_QUERIES_FOR_PUT):
             response = self.view(request, pk='1').render()
         self.assertEqual(response.status_code, status.HTTP_200_OK)
-        self.assertEqual(response.data, {'id': 1, 'text': 'foobar'})
+        self.assertEqual(dict(response.data), {'id': 1, 'text': 'foobar'})
         updated = self.objects.get(id=1)
         self.assertEqual(updated.text, 'foobar')
 
@@ -224,7 +218,7 @@ class TestInstanceView(TestCase):
         data = {'text': 'foobar'}
         request = factory.patch('/1', data, format='json')
 
-        with self.assertNumQueries(2):
+        with self.assertNumQueries(EXPECTED_QUERIES_FOR_PUT):
             response = self.view(request, pk=1).render()
         self.assertEqual(response.status_code, status.HTTP_200_OK)
         self.assertEqual(response.data, {'id': 1, 'text': 'foobar'})
@@ -243,89 +237,6 @@ class TestInstanceView(TestCase):
         ids = [obj.id for obj in self.objects.all()]
         self.assertEqual(ids, [2, 3])
 
-    def test_options_instance_view(self):
-        """
-        OPTIONS requests to RetrieveUpdateDestroyAPIView should return metadata
-        """
-        request = factory.options('/1')
-        with self.assertNumQueries(1):
-            response = self.view(request, pk=1).render()
-        expected = {
-            'parses': [
-                'application/json',
-                'application/x-www-form-urlencoded',
-                'multipart/form-data'
-            ],
-            'renders': [
-                'application/json',
-                'text/html'
-            ],
-            'name': 'Instance',
-            'description': 'Example description for OPTIONS.',
-            'actions': {
-                'PUT': {
-                    'text': {
-                        'max_length': 100,
-                        'read_only': False,
-                        'required': True,
-                        'type': 'string',
-                        'label': 'Text comes here',
-                        'help_text': 'Text description.'
-                    },
-                    'id': {
-                        'read_only': True,
-                        'required': False,
-                        'type': 'integer',
-                        'label': 'ID',
-                    },
-                }
-            }
-        }
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        self.assertEqual(response.data, expected)
-
-    def test_options_before_instance_create(self):
-        """
-        OPTIONS requests to RetrieveUpdateDestroyAPIView should return metadata
-        before the instance has been created
-        """
-        request = factory.options('/999')
-        with self.assertNumQueries(1):
-            response = self.view(request, pk=999).render()
-        expected = {
-            'parses': [
-                'application/json',
-                'application/x-www-form-urlencoded',
-                'multipart/form-data'
-            ],
-            'renders': [
-                'application/json',
-                'text/html'
-            ],
-            'name': 'Instance',
-            'description': 'Example description for OPTIONS.',
-            'actions': {
-                'PUT': {
-                    'text': {
-                        'max_length': 100,
-                        'read_only': False,
-                        'required': True,
-                        'type': 'string',
-                        'label': 'Text comes here',
-                        'help_text': 'Text description.'
-                    },
-                    'id': {
-                        'read_only': True,
-                        'required': False,
-                        'type': 'integer',
-                        'label': 'ID',
-                    },
-                }
-            }
-        }
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        self.assertEqual(response.data, expected)
-
     def test_get_instance_view_incorrect_arg(self):
         """
         GET requests with an incorrect pk type, should raise 404, not 500.
@@ -342,7 +253,7 @@ class TestInstanceView(TestCase):
         """
         data = {'id': 999, 'text': 'foobar'}
         request = factory.put('/1', data, format='json')
-        with self.assertNumQueries(2):
+        with self.assertNumQueries(EXPECTED_QUERIES_FOR_PUT):
             response = self.view(request, pk=1).render()
         self.assertEqual(response.status_code, status.HTTP_200_OK)
         self.assertEqual(response.data, {'id': 1, 'text': 'foobar'})
@@ -351,18 +262,15 @@ class TestInstanceView(TestCase):
 
     def test_put_to_deleted_instance(self):
         """
-        PUT requests to RetrieveUpdateDestroyAPIView should create an object
-        if it does not currently exist.
+        PUT requests to RetrieveUpdateDestroyAPIView should return 404 if
+        an object does not currently exist.
         """
         self.objects.get(id=1).delete()
         data = {'text': 'foobar'}
         request = factory.put('/1', data, format='json')
-        with self.assertNumQueries(3):
+        with self.assertNumQueries(1):
             response = self.view(request, pk=1).render()
-        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
-        self.assertEqual(response.data, {'id': 1, 'text': 'foobar'})
-        updated = self.objects.get(id=1)
-        self.assertEqual(updated.text, 'foobar')
+        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
 
     def test_put_to_filtered_out_instance(self):
         """
@@ -373,35 +281,7 @@ class TestInstanceView(TestCase):
         filtered_out_pk = BasicModel.objects.filter(text='filtered out')[0].pk
         request = factory.put('/{0}'.format(filtered_out_pk), data, format='json')
         response = self.view(request, pk=filtered_out_pk).render()
-        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
-
-    def test_put_as_create_on_id_based_url(self):
-        """
-        PUT requests to RetrieveUpdateDestroyAPIView should create an object
-        at the requested url if it doesn't exist.
-        """
-        data = {'text': 'foobar'}
-        # pk fields can not be created on demand, only the database can set the pk for a new object
-        request = factory.put('/5', data, format='json')
-        with self.assertNumQueries(3):
-            response = self.view(request, pk=5).render()
-        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
-        new_obj = self.objects.get(pk=5)
-        self.assertEqual(new_obj.text, 'foobar')
-
-    def test_put_as_create_on_slug_based_url(self):
-        """
-        PUT requests to RetrieveUpdateDestroyAPIView should create an object
-        at the requested url if possible, else return HTTP_403_FORBIDDEN error-response.
-        """
-        data = {'text': 'foobar'}
-        request = factory.put('/test_slug', data, format='json')
-        with self.assertNumQueries(2):
-            response = self.slug_based_view(request, slug='test_slug').render()
-        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
-        self.assertEqual(response.data, {'slug': 'test_slug', 'text': 'foobar'})
-        new_obj = SlugBasedModel.objects.get(slug='test_slug')
-        self.assertEqual(new_obj.text, 'foobar')
+        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
 
     def test_patch_cannot_create_an_object(self):
         """
@@ -414,6 +294,16 @@ class TestInstanceView(TestCase):
         self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
         self.assertFalse(self.objects.filter(id=999).exists())
 
+    def test_put_error_instance_view(self):
+        """
+        Incorrect PUT requests in HTML should include a form error.
+        """
+        data = {'text': 'foobar' * 100}
+        request = factory.put('/', data, HTTP_ACCEPT='text/html')
+        response = self.view(request, pk=1).render()
+        expected_error = '<span class="help-block">Ensure this field has no more than 100 characters.</span>'
+        self.assertIn(expected_error, response.rendered_content.decode('utf-8'))
+
 
 class TestFKInstanceView(TestCase):
     def setUp(self):
@@ -433,62 +323,16 @@ class TestFKInstanceView(TestCase):
         ]
         self.view = FKInstanceView.as_view()
 
-    def test_options_root_view(self):
-        """
-        OPTIONS requests to ListCreateAPIView should return metadata
-        """
-        request = factory.options('/999')
-        with self.assertNumQueries(1):
-            response = self.view(request, pk=999).render()
-        expected = {
-            'name': 'Fk Instance',
-            'description': 'FK: example description for OPTIONS.',
-            'renders': [
-                'application/json',
-                'text/html'
-            ],
-            'parses': [
-                'application/json',
-                'application/x-www-form-urlencoded',
-                'multipart/form-data'
-            ],
-            'actions': {
-                'PUT': {
-                    'id': {
-                        'type': 'integer',
-                        'required': False,
-                        'read_only': True,
-                        'label': 'ID'
-                    },
-                    'name': {
-                        'type': 'string',
-                        'required': True,
-                        'read_only': False,
-                        'label': 'name',
-                        'max_length': 100
-                    },
-                    'target': {
-                        'type': 'field',
-                        'required': True,
-                        'read_only': False,
-                        'label': 'Target',
-                        'help_text': 'Target'
-                    }
-                }
-            }
-        }
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        self.assertEqual(response.data, expected)
-
 
 class TestOverriddenGetObject(TestCase):
     """
     Test cases for a RetrieveUpdateDestroyAPIView that does NOT use the
     queryset/model mechanism but instead overrides get_object()
     """
+
     def setUp(self):
         """
-        Create 3 BasicModel intances.
+        Create 3 BasicModel instances.
         """
         items = ['foo', 'bar', 'baz']
         for item in items:
@@ -503,7 +347,7 @@ class TestOverriddenGetObject(TestCase):
             """
             Example detail view for override of get_object().
             """
-            model = BasicModel
+            serializer_class = BasicSerializer
 
             def get_object(self):
                 pk = int(self.kwargs['pk'])
@@ -561,11 +405,13 @@ class ClassB(models.Model):
 
 class ClassA(models.Model):
     name = models.CharField(max_length=255)
-    childs = models.ManyToManyField(ClassB, blank=True, null=True)
+    children = models.ManyToManyField(ClassB, blank=True, null=True)
 
 
 class ClassASerializer(serializers.ModelSerializer):
-    childs = serializers.PrimaryKeyRelatedField(many=True, source='childs')
+    children = serializers.PrimaryKeyRelatedField(
+        many=True, queryset=ClassB.objects.all()
+    )
 
     class Meta:
         model = ClassA
@@ -573,11 +419,11 @@ class ClassASerializer(serializers.ModelSerializer):
 
 class ExampleView(generics.ListCreateAPIView):
     serializer_class = ClassASerializer
-    model = ClassA
+    queryset = ClassA.objects.all()
 
 
-class TestM2MBrowseableAPI(TestCase):
-    def test_m2m_in_browseable_api(self):
+class TestM2MBrowsableAPI(TestCase):
+    def test_m2m_in_browsable_api(self):
         """
         Test for particularly ugly regression with m2m in browsable API
         """
@@ -603,7 +449,7 @@ class TwoFieldModel(models.Model):
 
 
 class DynamicSerializerView(generics.ListCreateAPIView):
-    model = TwoFieldModel
+    queryset = TwoFieldModel.objects.all()
     renderer_classes = (renderers.BrowsableAPIRenderer, renderers.JSONRenderer)
 
     def get_serializer_class(self):
@@ -612,12 +458,14 @@ class DynamicSerializerView(generics.ListCreateAPIView):
                 class Meta:
                     model = TwoFieldModel
                     fields = ('field_b',)
-            return DynamicSerializer
-        return super(DynamicSerializerView, self).get_serializer_class()
+        else:
+            class DynamicSerializer(serializers.ModelSerializer):
+                class Meta:
+                    model = TwoFieldModel
+        return DynamicSerializer
 
 
 class TestFilterBackendAppliedToViews(TestCase):
-
     def setUp(self):
         """
         Create 3 BasicModel instances to filter on.
@@ -660,7 +508,7 @@ class TestFilterBackendAppliedToViews(TestCase):
         request = factory.get('/1')
         response = instance_view(request, pk=1).render()
         self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
-        self.assertEqual(response.data, {'detail': 'Not found'})
+        self.assertEqual(response.data, {'detail': 'Not found.'})
 
     def test_get_instance_view_will_return_single_object_when_filter_does_not_exclude_it(self):
         """
@@ -681,3 +529,17 @@ class TestFilterBackendAppliedToViews(TestCase):
         response = view(request).render()
         self.assertContains(response, 'field_b')
         self.assertNotContains(response, 'field_a')
+
+
+class TestGuardedQueryset(TestCase):
+    def test_guarded_queryset(self):
+        class QuerysetAccessError(generics.ListAPIView):
+            queryset = BasicModel.objects.all()
+
+            def get(self, request):
+                return Response(list(self.queryset))
+
+        view = QuerysetAccessError.as_view()
+        request = factory.get('/')
+        with pytest.raises(RuntimeError):
+            view(request).render()
